From d3099e94a6c013769ae56ae377c80da8fa453d5d Mon Sep 17 00:00:00 2001
From: Valentin Longchamp <valentin.longchamp@epfl.ch>
Date: Wed, 1 Apr 2009 16:32:43 +0200
Subject: [PATCH] mx31moboard: camera support for marxbot board

This is an example of how the camera init function could be implemented.

Signed-off-by: Valentin Longchamp <valentin.longchamp@epfl.ch>
---
 arch/arm/mach-mx3/mx31moboard-marxbot.c |  104 ++++++++++++++++++++++++++++++-
 1 files changed, 103 insertions(+), 1 deletions(-)

diff --git a/arch/arm/mach-mx3/mx31moboard-marxbot.c b/arch/arm/mach-mx3/mx31moboard-marxbot.c
index 9ef9566..6b99b3b 100644
--- a/arch/arm/mach-mx3/mx31moboard-marxbot.c
+++ b/arch/arm/mach-mx3/mx31moboard-marxbot.c
@@ -18,20 +18,122 @@
 
 #include <linux/types.h>
 #include <linux/init.h>
-
+#include <linux/delay.h>
 #include <linux/platform_device.h>
 
+#include <media/soc_camera.h>
+
 #include <mach/hardware.h>
 #include <mach/common.h>
 #include <mach/imx-uart.h>
 #include <mach/iomux-mx3.h>
+#include <mach/gpio.h>
+#include <mach/ipu.h>
+#include <mach/mx3_camera.h>
 
 #include "devices.h"
 
+static int csi_pins[] = {
+	MX31_PIN_CSI_D4__CSI_D4, MX31_PIN_CSI_D5__CSI_D5,
+	MX31_PIN_CSI_D6__CSI_D6, MX31_PIN_CSI_D7__CSI_D7,
+	MX31_PIN_CSI_D8__CSI_D8, MX31_PIN_CSI_D9__CSI_D9,
+	MX31_PIN_CSI_D10__CSI_D10, MX31_PIN_CSI_D11__CSI_D11,
+	MX31_PIN_CSI_D12__CSI_D12, MX31_PIN_CSI_D13__CSI_D13,
+	MX31_PIN_CSI_D14__CSI_D14, MX31_PIN_CSI_D15__CSI_D15,
+	MX31_PIN_CSI_HSYNC__CSI_HSYNC, MX31_PIN_CSI_MCLK__CSI_MCLK,
+	MX31_PIN_CSI_PIXCLK__CSI_PIXCLK, MX31_PIN_CSI_VSYNC__CSI_VSYNC,
+	MX31_PIN_GPIO3_0__GPIO3_0, MX31_PIN_GPIO3_1__GPIO3_1,
+	MX31_PIN_TXD2__GPIO1_28,
+};
+
+#define CAM_RESET_B	IOMUX_TO_GPIO(MX31_PIN_GPIO3_0)
+#define CAM_STANDBY	IOMUX_TO_GPIO(MX31_PIN_GPIO3_1)
+#define CAM_MUX		IOMUX_TO_GPIO(MX31_PIN_TXD2)
+
+static int marxbot_mt9t031_power(struct device *dev, int on)
+{
+	/*GPIO3_0 is connected to standby, it would be more a sleep function*/
+	if (on)
+		gpio_set_value(CAM_STANDBY, 0);
+	else
+		gpio_set_value(CAM_STANDBY, 1);
+
+	return 0;
+}
+
+static int marxbot_mt9t031_reset(struct device *dev)
+{
+	pr_debug("Reset camera\n");
+	gpio_set_value(CAM_RESET_B, 0);
+	udelay(100);
+	gpio_set_value(CAM_RESET_B, 1);
+	return 0;
+}
+
+static struct soc_camera_link mt9t031_pdata = {
+	.bus_id	= 0,
+	.power	= marxbot_mt9t031_power,
+	.reset	= marxbot_mt9t031_reset,
+};
+
+#ifdef CONFIG_I2C_IMX
+static struct i2c_board_info marxbot_i2c_devices[] = {
+	[0] = {
+		I2C_BOARD_INFO("mt9t031", 0x5d),
+		.platform_data = &mt9t031_pdata,
+	},
+};
+#endif
+
+/*
+ * Try to reserve buffer space enough for 8 buffers 320x240@1 for
+ * streaming plus 2 buffers 2048x1536@1 for still image < 10MB
+ */
+#define CAMERA_MEM_SIZE (4 * 1024 * 1024)
+
+static struct mx3_camera_pdata camera_pdata = {
+	.flags 		= MX3_CAMERA_DATAWIDTH_8 | MX3_CAMERA_DATAWIDTH_10,
+	.mclk_10khz	= 4800,
+	.dma_dev	= &mx3_ipu.dev,
+};
+
+static int __init mx31moboard_csi_init(void)
+{
+	dma_addr_t dma_handle;
+	void *buf;
+
+	mxc_iomux_setup_multiple_pins(csi_pins, ARRAY_SIZE(csi_pins), "csi");
+
+	gpio_direction_output(CAM_MUX, 1);
+	gpio_export(CAM_MUX, false);
+	gpio_direction_output(CAM_RESET_B, 1);
+	gpio_direction_output(CAM_STANDBY, 0);
+
+	buf = dma_alloc_coherent(NULL, CAMERA_MEM_SIZE, &dma_handle, GFP_KERNEL);
+	if (!buf) {
+		pr_err("%s: cannot allocate camera buffer-memory\n", __func__);
+		return -ENOMEM;
+	}
+
+	memset(buf, 0, CAMERA_MEM_SIZE);
+	dma_declare_coherent_memory(&mx3_camera.dev,
+				dma_handle, dma_handle, CAMERA_MEM_SIZE,
+				DMA_MEMORY_MAP | DMA_MEMORY_EXCLUSIVE);
+
+	return mxc_register_device(&mx3_camera, &camera_pdata);
+}
+
 /*
  * system init for baseboard usage. Will be called by mx31moboard init.
  */
 void __init mx31moboard_marxbot_init(void)
 {
 	printk(KERN_INFO "Initializing mx31marxbot peripherals\n");
+
+	mx31moboard_csi_init();
+
+#ifdef CONFIG_I2C_IMX
+	i2c_register_board_info(0, marxbot_i2c_devices,
+			ARRAY_SIZE(marxbot_i2c_devices));
+#endif
 }
-- 
1.5.4

